<?php

namespace App\Service;

use Dompdf\Dompdf;
use Dompdf\Options;
use \Datetime;
use Twig\Environment;
use Twig\Loader\FilesystemLoader;

class Reports 
{
  private $util;
  private $orderService;

  public function __construct(\App\Util\Util $util, \App\Service\Order $orderService) {
    $this->util = $util;
    $this->orderService = $orderService;
  }

  public function exportPDF($params) {
    $newOrder = array();
    $title = $params['title'];
    $order = $this->orderService->history($params);
    $this->formatOrder($order, $newOrder);
    $html = $this->formatContent($title, $newOrder);

    $options = new Options();
    $options->set('isHtml5ParserEnabled', true);
    $options->set('isPhpEnabled', true);
    $options->setIsRemoteEnabled(true);

    $dompdf = new Dompdf($options);
    $dompdf->loadHtml($html);
    $dompdf->setPaper('A4', 'landscape');
    $dompdf->render();
    $pdfOutput = $dompdf->output();

    return $pdfOutput;
  }

  private function formatOrder($order, &$newOrder) {
    foreach ($order as $items) {
      $newOrder[] = array(
        'CDFILIAL'      => $items['CDFILIAL'],
        'CDLOJA'        => $items['CDLOJA'],
        'NRCOMANDA'     => $items['NRCOMANDA'],
        'NRPEDCENTRAL'  => $items['NRPEDCENTRAL'],
        'NRCOMANDAEXT'  => $items['NRCOMANDAEXT'],
        'DTHRABERMESA'  => $items['DTHRABERMESA'],
        'NMCONSUMIDOR'  => $items['NMCONSUMIDOR'],
        'IDORGCMDVENDA' => $this->util->commandSourceDictionary($items['IDORGCMDVENDA']),
        'STATUS'        => $this->util->formatStatus($items['STATUS'], $items['IDSINCMSDE']),
        'VRPEDIDO'      => $this->util->formatMoney($items['VRPEDIDO']),
        'IDSINCMSDE'    => $this->util->formatSynced($items['IDSINCMSDE']),
        'AGENDADO'      => $items['AGENDADO'],
        'TOGO'          => $items['TOGO'],
        'NRTELECONS'    => $items['NRTELECONS'],
        'QTDPROD'       => $items['QTDPROD'],
        'DSAREAATEND'   => $items['DSAREAATEND'],
      );
    }
  }

  private function formatContent($title, &$newOrder) {
    $loader = new FilesystemLoader(__DIR__ . '/../../templates');
    $twig = new Environment($loader);
    $template = $twig->load('reports.twig');

    $formattedContent = $template->render([
      'title' => $title,
      'datetime' => (new DateTime())->format('d/m/Y H:i:s'),
      'newOrder' => $newOrder,
    ]);

    return $formattedContent;
  }
}