import {
  AlertService,
  LoadingService, Modal, ModalService,
} from '@zeedhi/common';
import {
  Metadata, RestDatasource, Http,
} from '@zeedhi/core';
import { TekMemoryDatasource } from '@zeedhi/teknisa-components-common';
import { TekGrid } from '@zeedhi/teknisa-components-vuetify';
import { ZdTextInput, ZdTextarea } from '@zeedhi/vuetify';
import { Highlight } from '@zeedhi/zd-highlight';

type DataSourceTypes = (RestDatasource | TekMemoryDatasource);

type TekGridWithDataSource = TekGrid & {
  datasource?: DataSourceTypes
};

export class ErrosController {
  private modalConfirmDeleteErros!: Modal;

  private modalErros!: Modal;

  public async createModals() {
    this.modalConfirmDeleteErros = await ModalService.createFromJson('modal-confirm-delete-erros', true);
    this.modalErros = await ModalService.createFromJson('modal-erros', true);
  }

  public destroyModals() {
    this.modalConfirmDeleteErros?.destroy();
    this.modalErros?.destroy();
  }

  public openModalConfirmDeleteErros() {
    this.modalConfirmDeleteErros.show();
  }

  public closeModalConfirmDeleteErros() {
    this.modalConfirmDeleteErros.hide();
  }

  public closeModalErros() {
    this.modalErros.hide();
  }

  public async openModalErros({ row }: any) {
    let result;
    LoadingService.show('Carregando...');
    this.modalErros.show();

    const {
      CDINTEGRACAOERR, CDFILIAL, NMINTEGRACAO, DTINCLUSAO,
    } = row;

    try {
      result = await Http.post('/logTracking/logDetails', { CDINTEGRACAOERR });
      const jsonParse = JSON.parse(result.data.data);
      const jsonStringify = JSON.stringify(jsonParse, null, 2);
      const instance = Metadata.getInstance<Highlight>('errosDetailHighlight');
      instance.code = jsonStringify;
    } catch (error) {
      const instance = Metadata.getInstance<Highlight>('errosDetailHighlight');
      instance.code = 'Nenhum registro encontrado.';

      if (result.data.message) {
        AlertService.show({
          name: 'fail', text: result.data.message, right: true, top: true, color: 'red',
        });
      }
    } finally {
      LoadingService.hide();
    }

    const textarea = Metadata.getInstance<ZdTextarea>('errosDetailTextarea');
    const replace = row.DSINTEGRACAOERR.replace(/\\n/g, '\n');
    textarea.value = replace;

    const textCdFilial = Metadata.getInstance<ZdTextInput>('textCdFilial');
    textCdFilial.value = CDFILIAL;

    const textNmIntegracao = Metadata.getInstance<ZdTextInput>('textNmIntegracao');
    textNmIntegracao.value = NMINTEGRACAO;

    const textDtInclusao = Metadata.getInstance<ZdTextInput>('textDtInclusao');
    textDtInclusao.value = DTINCLUSAO;
  }

  public getInstancesFromModalConfirmDeleteErros(row: any) {
    const container = this.modalConfirmDeleteErros.children.find(
      (child: any) => child.name === 'confirmDeleteErrosContainer',
    );
    const form = container?.children?.find((child: any) => child.name === 'confirmDeleteform');

    form?.children?.map((child: any) => {
      child.value = row[child.name] || '';

      return child;
    });
  }

  public buttonDeleteErros({ event, row }: any) {
    event?.stopPropagation();
    this.openModalConfirmDeleteErros();
    this.getInstancesFromModalConfirmDeleteErros(row);
  }

  public getGrid() {
    const instance = Metadata.getInstance<TekGrid>('gridErros');
    return instance;
  }

  public getIntegracaoErrFromModalConfirmDeleteErros() {
    const container = this.modalConfirmDeleteErros.children.find(
      (child: any) => child.name === 'confirmDeleteErrosContainer',
    );
    const form = container?.children?.find((child: any) => child.name === 'confirmDeleteform');
    const textIntegracaoErr = form?.children?.find((child: any) => child.name === 'CDINTEGRACAOERR');
    const cdIntegracaoErr = textIntegracaoErr?.value;

    return cdIntegracaoErr;
  }

  public async deleteErros(row: any) {
    try {
      const CDINTEGRACAOERR = this.getIntegracaoErrFromModalConfirmDeleteErros();
      const { data } = await Http.post('/logTracking/deleteLogs', { CDINTEGRACAOERR });

      if (data.error) {
        throw new Error(`Falha ao excluir o Erro de Integração: ${data.message}`);
      }

      const grid: TekGridWithDataSource = this.getGrid();
      const findRowToDelete = (currentRow: any) => row.CDINTEGRACAOERR === currentRow.CDINTEGRACAOERR;

      const { data: gridData } = grid.datasource || {};

      if (Array.isArray(gridData)) {
        const idxRowToDelete = gridData.findIndex(findRowToDelete);

        gridData.splice(idxRowToDelete, 1);
        grid.datasource?.get();

        AlertService.show({
          name: 'deleteSuccess',
          text: 'Erro de Integração excluído com sucesso!',
          top: true,
          color: 'green',
          right: true,
        });
      }
    } catch (error) {
      this.closeModalErros();

      if (error instanceof Error) {
        AlertService.show({
          name: 'deleteFail', text: error.message, top: true, color: 'red',
        });
      }
    } finally {
      this.closeModalConfirmDeleteErros();
    }
  }
}
