import { FormatHandle } from '../interfaces/FormatHandle';
import { MoneyOptions } from '../interfaces/MoneyOptions';

class FormatMoney implements FormatHandle {
  public moneyDefault = {
    locale: 'pt-BR',
    currency: 'BRL',
  };

  public format(value: string | number, options: MoneyOptions): string | any {
    const optionsValues = { ...this.moneyDefault, ...options };

    return !this.isValid(optionsValues) && this.isValid(optionsValues) !== undefined
      ? this.isValid(optionsValues)
      : Intl.NumberFormat(optionsValues.locale, { style: 'currency', ...optionsValues }).format(Number(value));
  }

  public isValid(options: MoneyOptions): any {
    const locale = new RegExp('^([a-z]{2}-([A-Z]{2}))$');
    const currency = new RegExp('^([A-Z]{3})$');
    if (!locale.test(options.locale)) throw new Error(`Valor informado incorreto: ${options.locale}`);
    if (!currency.test(options.currency)) throw new Error(`Valor informado incorreto: ${options.currency}`);
  }
}

export default new FormatMoney();
